#include "helper.h"
#include<QDateTime>
#include<QSettings>
#include<QFileInfo>
#include<QDebug>
#include<QTextCodec>
#include<QProcess>
#include <QDir>
using namespace KAuth;
#define DATETIME_SERVICE             "org.freedesktop.timedate1"
#define DATETIME_DBUS_INTERFACE      "org.freedesktop.timedate1"
#define DATETIME_DBUS_PATH           "/org/freedesktop/timedate1"
#define WALLPAPER_PATH               "/usr/share/nde/wallpapers/"
#define USER_AVATAR                 "/var/lib/AccountsService/icons"
#define FONT_SCALE_FILE              "/etc/profile.d/hidpi.sh"
#define WALLPAPER_THUMBNAIL_PATH  "/usr/share/nde/wallpapers/convert/"

AuthHelper::AuthHelper():
    mDateTimeInterface(new QDBusInterface( DATETIME_SERVICE,
                                          DATETIME_DBUS_PATH,
                                          DATETIME_DBUS_INTERFACE,
                                          QDBusConnection::systemBus()) )
{
    qRegisterMetaType<QList<long long> >();

}
ActionReply AuthHelper::changecpuscheduler(const QVariantMap &parameters) {
    int pid = qvariant_cast<int>(parameters.value(QStringLiteral("pid")));
    int cpuScheduler = qvariant_cast<int>(parameters.value(QStringLiteral("cpuScheduler")));
    int cpuSchedulerPriority = qvariant_cast<int>(parameters.value(QStringLiteral("cpuSchedulerPriority")));
    bool success = true;

    struct sched_param param;
    param.sched_priority = cpuSchedulerPriority;
    if (sched_setscheduler(pid, cpuScheduler, &param) == -1) //设置优先级
    {
        perror("sched_setscheduler() failed");
        return false;
    }
    if(success)
        return ActionReply::SuccessReply();
    else
        return ActionReply(ActionReply::HelperErrorType);

}

ActionReply AuthHelper::setntp(const QVariantMap &parameters)
{
    chattr attr;
    bool enable = qvariant_cast<int>(parameters.value(QStringLiteral("enable")));
    QDBusMessage reply = mDateTimeInterface->call(QLatin1String("SetNTP"),enable,true);

    if(reply.type() != QDBusMessage::ErrorMessage){
        return ActionReply::SuccessReply();
    }else{
        return ActionReply::HelperErrorType;
    }
}

ActionReply AuthHelper::settimezone(const QVariantMap &parameters)
{
    chattr attr;
    QString tz = qvariant_cast<QString>(parameters.value(QStringLiteral("timezone")));
    QDBusMessage reply = mDateTimeInterface->call(QLatin1String("SetTimezone"),tz,true);

    if(reply.type() != QDBusMessage::ErrorMessage){
        return ActionReply::SuccessReply();
    }else{
        return ActionReply::HelperErrorType;
    }
}

ActionReply AuthHelper::setdatetime(const QVariantMap &parameters)
{
    chattr attr;
    QString dateime = qvariant_cast<QString>(parameters.value(QStringLiteral("datetime")));
    QDateTime t ;
    t = QDateTime::fromString(dateime,"yyyy-MM-dd hh:mm:ss");

    qint64 timestamp = t.toMSecsSinceEpoch() * 1000;
    QDBusMessage reply  = mDateTimeInterface->call("SetTime", timestamp, false, true);
    if(reply.type() != QDBusMessage::ErrorMessage){
        return ActionReply::SuccessReply();
    }else{
        return ActionReply::HelperErrorType;
    }
}
ActionReply AuthHelper::setdesktopfile(const QVariantMap &parameters)
{
    QString filename = qvariant_cast<QString>(parameters.value(QStringLiteral("filename")));
    QString s = qvariant_cast<QString>(parameters.value(QStringLiteral("Exec_value")));
    QFileInfo file(filename);
    if(file.exists() == false || s == "")
    {
        return ActionReply::HelperErrorReply();
    }
     QProcess p;
     p.execute( "bash",QStringList()<<"-c"<<QString("sed  -i  \'%1\'d  %2").arg("/Exec=/").arg(filename));
     p.waitForFinished();
     p.execute( "bash",QStringList()<<"-c"<<QString("sed -i  \'$a%1\'  %2").arg("Exec="+s).arg(filename));
     p.waitForFinished();
//    QSettings* setting = new QSettings(filename,QSettings::IniFormat);
//    setting->setIniCodec(QTextCodec::codecForName("UTF-8"));
//    setting->beginGroup("Desktop Entry");
//    setting->setValue("Exec",s);
//    setting->endGroup();
//    delete setting;
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::createuser(const QVariantMap &parameters)
{
    ActionReply reply;
    bool isSuccess = true;
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    QString password = qvariant_cast<QString>(parameters.value(QStringLiteral("password")));
    QString fullName = qvariant_cast<QString>(parameters.value(QStringLiteral("fullName")));
    QString accountType = qvariant_cast<QString>(parameters.value(QStringLiteral("accountType")));
    QString iconPath = qvariant_cast<QString>(parameters.value(QStringLiteral("iconPath")));
    UserAccount::AccountType type = UserAccount::AccountType::AdministratorAccountType;
    switch(accountType.toInt())
    {
    case 0:
        type = UserAccount::AccountType::StandardAccountType;
        break;
    case 1:
        type = UserAccount::AccountType::AdministratorAccountType;
        break;
    }

    AccountsManager *manager = new AccountsManager();
    if(!manager->createAUser(userName, "", type, password, iconPath, false)) {
        isSuccess = false;
    }
    delete manager;
    reply.addData("isSuccess", isSuccess);
    return reply;
}

ActionReply AuthHelper::setusername(const QVariantMap &parameters)
{
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    QString newUserName = qvariant_cast<QString>(parameters.value(QStringLiteral("newUserName")));
    AccountsManager *manager = new AccountsManager();
    if(!manager->editAUserName(userName, newUserName)) {
        delete manager;
        return ActionReply::HelperErrorType;
    }
    AvatarNRename(userName, newUserName);
    delete manager;
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::setpassword(const QVariantMap &parameters)
{
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    QString password = qvariant_cast<QString>(parameters.value(QStringLiteral("password")));

    AccountsManager *manager = new AccountsManager();
    if(!manager->editAUserPasswd(userName, true, password)) {
        delete manager;
        return ActionReply::HelperErrorType;
    }
    delete manager;
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::setaccounttype(const QVariantMap &parameters)
{
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    QString accountType = qvariant_cast<QString>(parameters.value(QStringLiteral("accountType")));
    UserAccount::AccountType type = UserAccount::AccountType::AdministratorAccountType;
    switch(accountType.toInt())
    {
    case 0:
        type = UserAccount::AccountType::AdministratorAccountType;
        break;
    case 1:
        type = UserAccount::AccountType::StandardAccountType;
        break;
    }

    AccountsManager *manager = new AccountsManager();
    if(!manager->editAUserType(userName, type)) {
        delete manager;
        return ActionReply::HelperErrorType;
    }
    delete manager;
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::deleteuser(const QVariantMap &parameters)
{
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    bool isDeleteHome = qvariant_cast<int>(parameters.value(QStringLiteral("deleteHome")));

    AccountsManager *manager = new AccountsManager();
    if(!manager->deleteUser(manager->findUserByName(userName), isDeleteHome)) {
        delete manager;
        return ActionReply::HelperErrorType;
    }
    delete manager;
    return ActionReply::SuccessReply();

}


ActionReply AuthHelper::setautologin(const QVariantMap &parameters) {
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    bool isAutoLogin = qvariant_cast<int>(parameters.value(QStringLiteral("autoLogin")));

    AccountsManager *manager = new AccountsManager();
    if(!manager->editAUserAutoLogin(userName, userName, isAutoLogin)) {
        delete manager;
        return ActionReply::HelperErrorType;
    }
    delete manager;
    return ActionReply::SuccessReply();
}
ActionReply AuthHelper::getlog(const QVariantMap &parameters)
{

    ActionReply reply;
    QString fileName = parameters["fileName"].toString();
    QFile file(fileName);
    if(!file.open(QIODevice::ReadOnly)) {
        reply = ActionReply::HelperErrorReply();
        reply.setErrorDescription(file.errorString());
        return reply;
    }
    QString contents(file.readAll());
    reply.addData("contents", contents);
    file.close();
    return reply;
}
ActionReply AuthHelper::getfiles(const QVariantMap &args) {
    ActionReply reply;
    QString dirName = args["dirName"].toString();
    QDir logDir(dirName);
    QFileInfoList logDirs = logDir.entryInfoList( QDir::Files | QDir::NoDotAndDotDot);
    QStringList dirList;
    for(const QFileInfo &info : logDirs) {
        dirList.append(info.absoluteFilePath());
    }
    reply.addData("Dirs", dirList);
    return reply;
}
ActionReply AuthHelper::rmfiles(const QVariantMap &parameters) {
    QStringList fileList = parameters["fileList"].toStringList();
    ActionReply reply = ActionReply::SuccessReply();
    QStringList failedDirList;
    qWarning() << fileList;
    for (const QString& file : fileList) {
        QFileInfo info(file);
        if(info.isDir())
        {
            QDir dir(file);
            if(!dir.removeRecursively())
            {
                qWarning() <<" remove " << file <<" failed";
                reply = ActionReply::HelperErrorReply();
                reply.setErrorDescription(file);
                failedDirList.removeOne(file);
            }
        }else if(info.isFile())
        {
            QFile logFile(file);
            if(!logFile.remove())
            {
                qWarning() <<" remove " << file <<" failed";
                reply = ActionReply::HelperErrorReply();
                reply.setErrorDescription(file);
                failedDirList.removeOne(file);
            }
        }
    }
    reply.addData("dirList", failedDirList);
    return reply;
}
ActionReply AuthHelper::compresslogs(const QVariantMap &parameters) {
    QStringList fileList = parameters["fileList"].toStringList();
    QString filename = parameters["compressFile"].toString();
    QString compressFile = filename + ".tar.gz";
    QProcess process;
    process.setWorkingDirectory("/var/log");
    process.start("tar", QStringList {} << "czf" << compressFile << fileList);
    process.waitForFinished();
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::setavatar(const QVariantMap &parameters) {
    QString userName = qvariant_cast<QString>(parameters.value(QStringLiteral("userName")));
    QString iconPath = qvariant_cast<QString>(parameters.value(QStringLiteral("iconPath")));

    AccountsManager *manager = new AccountsManager();
    UserAccount *findOutUser = manager->findUserByName(userName);
    if(findOutUser == nullptr) {
        QFileInfo file(iconPath);
        if(file.exists() == false)
        {
            return ActionReply::HelperErrorReply();
        }
        if(iconPath.left(iconPath.lastIndexOf("/")) == QString("/var/lib/AccountsService/images") || iconPath.left(iconPath.lastIndexOf("/")) == QString("/var/lib/AccountsService/images/local")) {
            QFile::copy(iconPath,USER_AVATAR+userName);
        }
        else {
            QFile::copy(iconPath,USER_AVATAR + userName);
            QFile::copy(iconPath,QString("/var/lib/AccountsService/images/local/") + userName);
        }
    }
    else if(!manager->editAUserIcon(userName, iconPath)) {
        delete manager;
        return ActionReply::HelperErrorType;
    }
    delete manager;
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::pictureschanged(const QVariantMap &parameters)
{
    QString filepath = qvariant_cast<QString>(parameters.value(QStringLiteral("filepath")));
    QString type = qvariant_cast<QString>(parameters.value(QStringLiteral("type")));
    QFileInfo file(filepath);
    ActionReply reply;
    bool isReplyEnable = false;
    if(file.exists() == false || type == "")
    {
        return ActionReply::HelperErrorReply();
    }
    if(type == "ADD"){
         int index = filepath.lastIndexOf("/");
         QString name = filepath.right(filepath.length()-index-1);
         QString itemPath = WALLPAPER_PATH + name;
         if(QFileInfo(itemPath).exists()){
//             perror("file already exists");
//             return ActionReply::HelperErrorReply();
             name = WallpaperRename(name);
             reply.addData("names", name);
             isReplyEnable = true;
         }
         bool res = true;
         bool res_thu = true;
         QDir dir;
         if(!dir.exists(WALLPAPER_PATH))
         {
            res = dir.mkpath(WALLPAPER_PATH);
            qDebug()<<"目录是否创建成功"<<res;
          }
         if(res)
         {
                QFile::copy(filepath,WALLPAPER_PATH+name);
         }
         else {
             perror("dir is not exists");
             return ActionReply::HelperErrorReply();
         }
         if(!dir.exists(WALLPAPER_THUMBNAIL_PATH))
         {
            res_thu = dir.mkpath(WALLPAPER_THUMBNAIL_PATH);
            qDebug()<<"目录是否创建成功"<<res;
          }
         if(res_thu)
         {
                QProcess p;
                QFile::copy(filepath,WALLPAPER_THUMBNAIL_PATH+name);
                QString path = WALLPAPER_THUMBNAIL_PATH+name;
                p.execute( "bash",QStringList()<<"-c"<<QString("convert -resize 190x130 -quality 100 -antialias -density 72 %1 %2").arg(path).arg(path));
                p.waitForFinished();
         }
         else {
             perror("dir is not exists");
             return ActionReply::HelperErrorReply();
         }
    }else if(type == "DEL"){
        int index = filepath.lastIndexOf("/");
        QString name = filepath.right(filepath.length()-index-1);
        QString itemPath = WALLPAPER_PATH + name;
        QFile::remove(itemPath);
        QFile::remove(filepath);
    }else {
        return ActionReply::HelperErrorReply();
    }
//    return ActionReply::SuccessReply();
    if(isReplyEnable){
          return reply;
    }
    else
    {
       return ActionReply::SuccessReply();
    }

}

QString AuthHelper::WallpaperRename(QString oldName)
{
    QDateTime dateTime(QDateTime::currentDateTime());
    QString qStr = dateTime.toString("yyyyMMddhhmmss");
    QString PreSuffixName = oldName.section('.',0,0);
    QString SuffixName = oldName.section('.',1,1);
    QString newName = PreSuffixName + '_' + qStr + '.' + SuffixName;

    return newName;
}

ActionReply AuthHelper::changehostname(const QVariantMap &parameters)
{
    QString hostname = qvariant_cast<QString>(parameters.value(QStringLiteral("hostname")));
    if(hostname.isEmpty())
    {
        return ActionReply::HelperErrorReply();
    }
    QProcess p;
    p.execute( "bash",QStringList()<<"-c"<<QString("hostnamectl set-hostname %1").arg(hostname));
    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::changesettingvalue(const QVariantMap &parameters)
{
    QString filename = qvariant_cast<QString>(parameters.value(QStringLiteral("filename")));
    QString k = qvariant_cast<QString>(parameters.value(QStringLiteral("key")));
    QString v = qvariant_cast<QString>(parameters.value(QStringLiteral("value")));
    QFileInfo file(filename);
    if(file.exists() == false || k.isEmpty())
    {
        return ActionReply::HelperErrorReply();
    }

    QSettings setting(filename,QSettings::IniFormat);
    setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
    setting.setValue(k,v);
    setting.sync();

    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::changeavatarname(const QVariantMap &parameters)
{
    QString oldName = qvariant_cast<QString>(parameters.value(QStringLiteral("oldName")));
    QString newName = qvariant_cast<QString>(parameters.value(QStringLiteral("newName")));
    AvatarNRename(oldName, newName);

    return ActionReply::SuccessReply();
}

void AuthHelper::AvatarNRename(QString oldName, QString newName)
{
    
    QString oldIconPath = QString("/var/lib/AccountsService/icons/") + oldName;
    QString newIconPath = QString("/var/lib/AccountsService/icons/") + newName;
    QFileInfo iconFile(oldIconPath);
    if(iconFile.exists())
    {
        QFile::rename(oldIconPath, newIconPath); // 重命名头像名
    }
    else
    {
        newIconPath = "";
    }
    
    
    QString newSettingPath = QString("/var/lib/AccountsService/users/") + newName;
    QFileInfo settingPath(newSettingPath);
    if(settingPath.exists())
    {
        QSettings setting(newSettingPath,QSettings::IniFormat);
        setting.setIniCodec(QTextCodec::codecForName("UTF-8"));
        setting.beginGroup("User");
        setting.setValue("Icon", newIconPath);
        setting.endGroup();
        setting.sync();
    }
}

ActionReply AuthHelper::installfont(const QVariantMap &parameters)
{
    QString sourceFont = qvariant_cast<QString>(parameters.value(QStringLiteral("sourceFont")));
    QString dirFont = QString("/usr/share/fonts/");
    QFileInfo file(sourceFont);
    if(file.exists() == false)
    {
        return ActionReply::HelperErrorReply();
    }
    QProcess p;
     p.execute( "bash",QStringList()<<"-c"<<QString("cp  -f  %1 %2").arg(sourceFont).arg(dirFont));
     p.waitForFinished();

    return ActionReply::SuccessReply();
}

ActionReply AuthHelper::setscaling(const QVariantMap &parameters)
{
    QString s = qvariant_cast<QString>(parameters.value(QStringLiteral("FONT_SCALE_LEVEL")));
    QFileInfo fileinfo(FONT_SCALE_FILE);
    if(fileinfo.exists() == false || s == "")
    {
        return ActionReply::HelperErrorReply();
    }
    QFile file(FONT_SCALE_FILE);
    QString str;
    if(file.open(QIODevice::ReadOnly|QIODevice::Text))
    {
        QString content = file.readAll();
        QStringList contentList = content.split("\n");
        for(QString line : contentList)
        {
            if("" == line)
                continue;
            if(line.contains("FONT_SCALE_LEVEL="))
            {
                 str +=   QString("FONT_SCALE_LEVEL=") + s +QString("\n");

            }
            else
                  str += line +QString("\n");
        }
        file.close();
        if(file.open(QIODevice::WriteOnly|QIODevice::Text))
        {
            QTextStream in(&file);
            in << str;
            file.close();
            return ActionReply::SuccessReply();
        }
        else
            return ActionReply::HelperErrorReply();
    }
    else
             return ActionReply::HelperErrorReply();
}

KAUTH_HELPER_MAIN("org.nde.auth.helper", AuthHelper)

